/* Copyright (c) 2007..2010 Dirk Jagdmann <doj@cubic.org>

This software is provided 'as-is', without any express or implied
warranty. In no event will the authors be held liable for any damages
arising from the use of this software.

Permission is granted to anyone to use this software for any purpose,
including commercial applications, and to alter it and redistribute it
freely, subject to the following restrictions:

    1. The origin of this software must not be misrepresented; you
       must not claim that you wrote the original software. If you use
       this software in a product, an acknowledgment in the product
       documentation would be appreciated but is not required.

    2. Altered source versions must be plainly marked as such, and
       must not be misrepresented as being the original software.

    3. This notice may not be removed or altered from any source
       distribution. */

/* $Header: /code/rndls/rndls.cpp,v 1.12 2010/02/09 02:14:41 doj Exp $ */

#include <iostream>
#include <string>
#include <vector>
#include <set>
#include <cstdlib>
#include <unistd.h>
#include <sys/types.h>
#include <dirent.h>

#include <path.hpp>
#include <rand.hpp>
#include <foreach.hpp>

typedef std::set<std::string> ext_t;

typedef std::vector<std::string> dirs_t;
dirs_t dirs, files;

void finddirs(std::string p)
{
  p=doj::absFilename(p);
  if(doj::isFile(p))
    {
      files.push_back(p);
      return;
    }
  if(!doj::isDir(p))
    return;

  DIR *dir=opendir(p.c_str());
  if(!dir)
    perror((std::string("could not opendir ")+p).c_str());
  struct dirent *d;
  while((d=readdir(dir)))
    {
      std::string D=d->d_name;
      if(D==".") continue;
      if(D=="..") continue;
      D=p+'/'+D;
      if(doj::isDir(D))
	finddirs(D);
    }
  closedir(dir);

  dirs.push_back(p);
}

void help()
{
  std::cout << "usage:   rndls [-0] [-e ext]* [-n number] <dirs|files>\n";
  std::cout << "version: 1.2\n";
}

int main(int argc, char **argv)
{
  doj::srand();

  std::string pwd=getenv("PWD");
  if(pwd.empty())
    {
      char buf[PATH_MAX];
      getcwd(buf, sizeof(buf));
      pwd=buf;
    }
  if(pwd.empty())
    {
      std::cerr << "could not get current work directory\n";
      return 1;
    }

  int i;
  int verbose=0;
  unsigned num=10;
  ext_t extensions;
  bool print0=false;
  while((i=getopt(argc, argv, "0e:hn:v")) >= 0)
    switch(i)
      {
      case '0': print0=true; break;
      case 'e': extensions.insert(optarg); break;

      case 'h':
      case '?':
	help();
	return 1;

      case 'n':
	num=strtoul(optarg, 0, 0);
	if(num<=0)
	  return 1;
	break;

      case 'v': verbose++; break;
      }

  if(optind==argc)
    finddirs(".");
  else
    for(i=optind; i<argc; ++i)
      finddirs(argv[i]);

  foreach(dirs, D)
    {
      if(verbose)
	std::clog << "DIR:  " << *D << std::endl;

      DIR *dir=opendir(D->c_str());
      if(dir)
	{
	  struct dirent *d;
	  while((d=readdir(dir)))
	    {
	      std::string F=d->d_name;
	      if(F==".") continue;
	      if(F=="..") continue;
	      F=*D+'/'+F;
	      if(doj::isFile(F))
		{
		  if(extensions.size())
		    {
		      foreach(extensions, i)
			if(F.rfind(*i) == F.size()-i->size())
			  files.push_back(F);
		    }
		  else
		    files.push_back(F);
		}
	    }
	  closedir(dir);
	}
    }

  if(files.empty())
    return 0;

  ext_t s1, s2;

  foreach(files, i)
    {
      s1.insert(*i);
      if(verbose)
	std::clog << "FILE: " << *i << std::endl;
    }

  while(num)
    {
      if(s1.empty())
	s1.swap(s2);

      ext_t::iterator it=s1.begin();
      advance(it, static_cast<unsigned>(doj::frand(s1.size()-1)) );
      const std::string fn=*it;
      s1.erase(fn);
      s2.insert(fn);

      std::cout << fn;
      if(print0)
	std::cout << '\0';
      else
	std::cout << std::endl;

      --num;
    }

  return 0;
}
